<?php

namespace SM\Service;

use SM\Factory\BillingComparisonFactory;
use SM\Service\FindService;
use Teknisa\Libs\Util\Environment;
use Teknisa\Libs\Util\NewCode;
use Doctrine\ORM\EntityManager;

class BillingComparisonService
{
    protected $BillingComparisonFactory;
    protected $findService;
    protected $Environment;
    protected $newCode;

    public function __construct(EntityManager $entityManager, BillingComparisonFactory $BillingComparisonFactory, FindService $findService, Environment $Environment, NewCode $newCode)
    {
        $this->connection               = $entityManager->getConnection();
        $this->BillingComparisonFactory = $BillingComparisonFactory;
        $this->FindService              = $findService;
        $this->Environment              = $Environment;
        $this->newCode                  = $newCode;
    }

    public function getBillingComparison($parametros)
    {
        if (isset($parametros["QTPARCELAS"])) {
            // formatando os parametros do contrato para _019_1092_1982
            $contratos = null;
            if (isset($parametros["NRSEQCONTRATO"])) {
                $contratos = $this->FindService->formatarArrayUndescore($parametros["NRSEQCONTRATO"]);
            }

            $parceiro = null;
            if (isset($parametros["NRPARCNEGOCIO"])) {
                $parceiro = $this->FindService->formatarArrayUndescore($parametros["NRPARCNEGOCIO"]);
            }

            $tipoorcamento = null;
            if (isset($parametros["NRTIPOORCAMENTO"])) {
                $tipoorcamento = $this->FindService->formatarArrayUndescore($parametros["NRTIPOORCAMENTO"]);
            }

            $idtpfatura = null;
            if (isset($parametros["IDTPFATURA"])) {
                $idtpfatura = $this->FindService->formatarArrayUndescore($parametros["NRTIPOORCAMENTO"]);
            }
        
            $validaPeriodoAtual = '';
            $validaPeriodoComp = '';
        
            if (isset($parametros['VALIDAPERIODOATUAL'])) {
                $validaPeriodoAtual = ($parametros['VALIDAPERIODOATUAL'] == 'N') ? ' AND PERIODO IS NOT NULL' : null;
            }
        
            if (isset($parametros['VALIDAPERIODOCOMP'])) {
                $validaPeriodoComp = ($parametros['VALIDAPERIODOCOMP'] == 'N') ? ' AND PERIODO_COMP IS NOT NULL' : null;
            }

            $registrosVerificados = '';
            if (isset($parametros['VERIFICARREGISTROS'])) {
                $registrosVerificados = ($parametros['VERIFICARREGISTROS'] == 'T') ? '' : (($parametros['VERIFICARREGISTROS'] == 'S') ?  " AND ((IDREGVERIFICADO = 'S') AND (IDREGVERIFICADO_COMP = 'S'))" : " AND ((IDREGVERIFICADO = 'N') OR (IDREGVERIFICADO_COMP = 'N'))");
            }
        
            return $this->BillingComparisonFactory->getBillingComparison(array(
                "MESCOMPET"           => $parametros["MESCOMPET"],
                "QTPARCELAS"          => $parametros["QTPARCELAS"],
                "NRSEQCONTRATO"       => $contratos,
                "NRPARCNEGOCIO"       => $parceiro,
                "NRTIPOORCAMENTO"     => $tipoorcamento,
                "IDTPFATURA"          => $idtpfatura,
                "VRPERCDIFERENCA_DE"  => isset($parametros["VRPERCDIFERENCA_DE"]) ? $parametros["VRPERCDIFERENCA_DE"] : null,
                "VRPERCDIFERENCA_ATE" => isset($parametros["VRPERCDIFERENCA_ATE"]) ? $parametros["VRPERCDIFERENCA_ATE"] : null,
                "VRDIFERENCA_DE"      => isset($parametros["VRDIFERENCA_DE"]) ? $parametros["VRDIFERENCA_DE"] : null,
                "VRDIFERENCA_ATE"     => isset($parametros["VRDIFERENCA_ATE"]) ? $parametros["VRDIFERENCA_ATE"] : null,
                "IDTIPO"              => $parametros["IDTIPO"]
            ), $validaPeriodoAtual, $validaPeriodoComp, $registrosVerificados);
        }
    }

    public function verificarCompFaturamento($parametros)
    {
        $this->connection->beginTransaction();
       
        try {
            $cdOperador = $this->Environment->getCdOperador();
            $nrOrg = $this->Environment->getNrOrg();
    
            $array = array();
            array_push($array, $parametros['CODIGO'], $parametros['CODIGO_COMP']);
            
            $arrayFormatado = $this->FindService->formatarArray($array);
    
            $param = array(
                "CDOPERULTATU"      => $cdOperador,
                "NRORGULTATU"       => $nrOrg,
            );

            if ($parametros['IDTIPO'] == 'F') {
                $this->BillingComparisonFactory->verificarRegParcela($param, $arrayFormatado);
            } elseif ($parametros['IDTIPO'] == 'M') {
                $this->BillingComparisonFactory->verificarRegMedicao($param, $arrayFormatado);
            }

            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    public function getContractsBillingComparison()
    {
        return $this->BillingComparisonFactory->getContractsBillingComparison();
    }

    public function getBusinessPartnersBilling()
    {
        return $this->BillingComparisonFactory->getBusinessPartnersBilling();
    }

    public function getBillingScopes()
    {
        return $this->BillingComparisonFactory->getBillingScopes();
    }

    public function billingComparisonOfTheParcelTab($parametros)
    {
        $param = [
            "NRFATURACONT" => $parametros["NR"],
        ];

        return $this->BillingComparisonFactory->billingComparisonOfTheParcelTab(
            $param
        );
    }

    public function billingComparisonOfMeasurementItems($parametros)
    {
        $param = [
            "NRMEDICAOFATURA" => $parametros["NR"],
        ];

        return $this->BillingComparisonFactory->billingComparisonOfMeasurementItems(
            $param
        );
    }

    public function billingComparisonOfVariableMeasures($parametros)
    {
        $param = [
            "NRMEDICAOFATURA" => $parametros["NR"],
        ];

        return $this->BillingComparisonFactory->billingComparisonOfVariableMeasures(
            $param
        );
    }
}
