import {
    Http,
    IDictionary,
    IEventParam,
    Metadata,
} from '@zeedhi/core';
import {
    AlertService, Checkbox, DialogService, Form, LoadingService, Modal, ModalService, Month, Number, Select, SelectMultiple, Tabs, Text, TextInput,
} from '@zeedhi/common';
import { TekGrid } from '@zeedhi/teknisa-components-common';
import { Utilities } from '@/util/Utilities';

export class ComparativoFaturamentoController {
    private modalFilter!: Modal;

    public visibility: boolean = true;

    public visibilityParcel: boolean = false;

    public visibilityMeasurement: boolean = false;

    public filterRow: any = {};

    public desabilitarCheck = false;

    get visibleFrame() {
        return !this.visibility;
    }

    public openPreview({ row }: any) {
        const setOfTabs = Metadata.getInstance<Tabs>('tabs-billingComparisonDetail');
        const checkbox = Metadata.getInstance<Checkbox>('checkbox-verificacao');

        this.visibility = false;

        window.localStorage.setItem('ROWDATA', JSON.stringify(row));

        this.visibilityParcel = (row.NRFATURACONT !== null || row.NRFATURACONT_COMP !== null);
        this.visibilityMeasurement = (row.NRMEDICAOFATURA !== null || row.NRMEDICAOFATURA_COMP !== null);

        if (row.IDREGVERIFICADO === 'S' && row.IDREGVERIFICADO_COMP === 'S') {
            checkbox.value = true;
            this.desabilitarCheck = true;
        } else {
            checkbox.value = null;
            this.desabilitarCheck = false;
        }
    
        setOfTabs.activeTab = this.visibilityParcel ? 0 : 1;

        this.reloadTabs();
    }

    public backToPageMain() {
        // Router.replace(`/informacoes_parceiros?${row.NRPARCNEGOCIO}`);
        // Router.replace('/billingComparison');
        this.visibility = true;
    }

    public eventFilter({ event }: any) {
        event.preventDefault();
        this.openModal();
    }

    public confirmarVerificacaoComparativo() {
        DialogService.show({
            name: 'confirmacao-dialog',
            type: 'warning',
            title: 'Atenção!',
            text: 'Você verificou este Comparativo de Faturamento? <br>Esta ação depois de realizada não poderá ser desfeita.',
            buttons: [
                {
                    name: 'nao-confirmar',
                    component: 'ZdButton',
                    label: 'Não',
                    events: {
                        click: () => {
                            this.limparMarcacao();
                            DialogService.hide();
                        },
                    },
                    outline: true,
                },
                {
                    name: 'confirmar',
                    component: 'ZdButton',
                    label: 'Sim',
                    events: {
                        click: () => {
                            this.verificarComparativoFaturamento();
                            DialogService.hide();
                        },
                    },
                },
            ],
        });
    }

    public limparMarcacao() {
        Metadata.getInstance<Checkbox>('checkbox-verificacao').value = null;
    }

    public verificarComparativoFaturamento() {
        const grid = Metadata.getInstance<TekGrid>('tekGrid-billingComparison');

        Http.post('/verificarCompFaturamento', {
            CODIGO: Metadata.getInstance<TextInput>('codigo').value,
            CODIGO_COMP: Metadata.getInstance<TextInput>('codigo-comparacao').value,
            IDTIPO: Metadata.getInstance<TextInput>('idTipo').value,
        }).then(() => {
            Utilities.showSuccessMessage();
            this.desabilitarCheck = true;
            grid.datasource.get();
        }).catch((error: any) => {
            Utilities.showErrorMessage(error);
        }).finally(() => {
            LoadingService.hide();
        });
    }

    public async openModal() {
        this.modalFilter = await ModalService.createFromJson('/billing/modal/modal-filter-billing', true);
        await this.modalFilter.show();

        const dataAtual = new Date();
        let mesAnterior = dataAtual.getMonth();
        let anoAnterior = dataAtual.getFullYear();
        if (mesAnterior === 0) {
            mesAnterior = 12;
            anoAnterior -= 1;
        }

        const mesAnteriorFormatado = `${mesAnterior.toString().padStart(2, '0')}/${anoAnterior}`;

        Metadata.getInstance<SelectMultiple>('select-IDTIPO').value = this.filterRow['select-IDTIPO'] || 'F';
        Metadata.getInstance<SelectMultiple>('select-contracts').value = this.filterRow['select-contracts'];
        Metadata.getInstance<Month>('period-month').value = this.filterRow['period-month'] || mesAnteriorFormatado;
        Metadata.getInstance<TextInput>('input-qtdParcelas').value = this.filterRow['input-qtdParcelas'] || '1';
        Metadata.getInstance<SelectMultiple>('select-bPartner').value = this.filterRow['select-bPartner'];
        Metadata.getInstance<SelectMultiple>('select-billingScope').value = this.filterRow['select-billingScope'];
        Metadata.getInstance<SelectMultiple>('select-billingType').value = this.filterRow['select-billingType'];
        Metadata.getInstance<SelectMultiple>('percentualDiferenca-de').value = this.filterRow['percentualDiferenca-de'];
        Metadata.getInstance<SelectMultiple>('percentualDiferenca-ate').value = this.filterRow['percentualDiferenca-ate'];
        Metadata.getInstance<Select>('validaPeriodoAtual').value = this.filterRow['validaPeriodo-Atual'] || 'S';
        Metadata.getInstance<Select>('validaPeriodoComp').value = this.filterRow['validaPeriodo-Comp'] || 'S';
        Metadata.getInstance<Number>('valor-diferenca-de').value = this.filterRow['valor-diferenca-de'];
        Metadata.getInstance<Number>('valor-diferenca-ate').value = this.filterRow['valor-diferenca-ate'];
        Metadata.getInstance<Select>('verificacao-registros').value = this.filterRow['verificacao-registros'] || 'T';
    }

    public closeModal() {
        this.modalFilter.hide();
        this.modalFilter?.destroy();
    }

    public filterGrid() {
        const form = Metadata.getInstance<Form>('form-filter');
        if (form.validate()) {
            const tipo = Metadata.getInstance<SelectMultiple>('select-IDTIPO').value;
            const contract = Metadata.getInstance<SelectMultiple>('select-contracts').value;
            // const date = Metadata.getInstance<DateRange>('period-dateRange').value;
            const date = Metadata.getInstance<Month>('period-month').value;
            let qtd = Metadata.getInstance<TextInput>('input-qtdParcelas').value;
            const parnegocio = Metadata.getInstance<SelectMultiple>('select-bPartner').value;
            const billingScope = Metadata.getInstance<SelectMultiple>('select-billingScope').value;
            const billingType = Metadata.getInstance<SelectMultiple>('select-billingType').value;
            const validaPeriodoAtual = Metadata.getInstance<Select>('validaPeriodoAtual').value;
            const validaPeriodoComp = Metadata.getInstance<Select>('validaPeriodoComp').value;
            const percentualDiferencaDe = Metadata.getInstance<Number>('percentualDiferenca-de').value;
            const percentualDiferencaAte = Metadata.getInstance<Number>('percentualDiferenca-ate').value;
            const vrDiferencaDe = Metadata.getInstance<Number>('valor-diferenca-de').value;
            const vrDiferencaAte = Metadata.getInstance<Number>('valor-diferenca-ate').value;
            const verRegistros = Metadata.getInstance<Select>('verificacao-registros').value;

            this.filterRow = {
                'select-IDTIPO': tipo,
                'select-contracts': contract,
                'period-month': date,
                'input-qtdParcelas': qtd,
                'select-bPartner': parnegocio,
                'select-billingScope': billingScope,
                'select-billingType': billingType,
                'validaPeriodo-Atual': validaPeriodoAtual,
                'validaPeriodo-Comp': validaPeriodoComp,
                'percentualDiferenca-de': percentualDiferencaDe,
                'percentualDiferenca-ate': percentualDiferencaAte,
                'valor-diferenca-de': vrDiferencaDe,
                'valor-diferenca-ate': vrDiferencaAte,
                'verificacao-registros': verRegistros,
            };

            const grid = Metadata.getInstance<TekGrid>('tekGrid-billingComparison');

            grid.getColumn('IDSITUACAOFAT').isVisible = tipo === 'F';
            grid.getColumn('IDSITUACAOFAT_COMP').isVisible = tipo === 'F';
            grid.getColumn('IDSITUACAOMED').isVisible = tipo === 'M';
            grid.getColumn('IDSITUACAOMED_COMP').isVisible = tipo === 'M';

            qtd = parseInt(qtd.replace('º', ''), 10);

            this.setFilterGridPrincipalComparativo(tipo, contract, date, qtd, parnegocio, billingScope, billingType, percentualDiferencaDe, percentualDiferencaAte, validaPeriodoAtual, validaPeriodoComp, vrDiferencaDe, vrDiferencaAte, verRegistros);
            this.closeModal();
        }
    }

    public setFilterGridPrincipalComparativo(tipo: string, contracts: any, date: any, qtd: number, parnegocio: any, billingScope: any, billingType: any, percentualDiferencaDe: any, percentualDiferencaAte: any, validaPeriodoAtual: any, validaPeriodoComp: any, vrDiferencaDe: any, vrDiferencaAte: any, verRegistros: any) {
        const grid = Metadata.getInstance<TekGrid>('tekGrid-billingComparison');

        grid.datasource.setFilter({
            IDTIPO: tipo,
            MESCOMPET: date,
            NRSEQCONTRATO: contracts,
            QTPARCELAS: qtd,
            NRPARCNEGOCIO: parnegocio,
            NRTIPOORCAMENTO: billingScope,
            IDTPFATURA: billingType,
            VRPERCDIFERENCA_DE: percentualDiferencaDe,
            VRPERCDIFERENCA_ATE: percentualDiferencaAte,
            VALIDAPERIODOATUAL: validaPeriodoAtual,
            VALIDAPERIODOCOMP: validaPeriodoComp,
            VRDIFERENCA_DE: vrDiferencaDe,
            VRDIFERENCA_ATE: vrDiferencaAte,
            VERIFICARREGISTROS: verRegistros,
        });
    }

    public alertMessage(message: string, type: string) {
        AlertService.show({
            name: 'basic-alert',
            text: message,
            top: true,
            left: false,
            timeout: 3000,
            color: type,
        });
    }

    public changeMaxWidth({ component }: any) {
        window.localStorage.removeItem('ROWDATA');

        component.columns[6].maxWidth = '250px';
        component.columns[6].overflow = 'wrap';
        component.columns[7].maxWidth = '200px';
        component.columns[7].overflow = 'wrap';
        component.columns[8].maxWidth = '250px';
        component.columns[8].overflow = 'wrap';
    }

    public onMountedHeaderBilling() {
        const rowdata = window.localStorage.getItem('ROWDATA');
        if (rowdata) {
            const data = JSON.parse(rowdata);
            const TIPODEFATURAMENTO = this.traduction(data.IDTPFATURA);

            Metadata.getInstance<Text>('text-NRFATURACONT').text = data.NRFATURACONT;
            Metadata.getInstance<Text>('text-NRFATURACONT_COMP').text = data.NRFATURACONT_COMP;
            Metadata.getInstance<Text>('text-NRMEDICAOFATURA').text = data.NRMEDICAOFATURA;
            Metadata.getInstance<Text>('text-NRMEDICAOFATURA_COMP').text = data.NRMEDICAOFATURA_COMP;

            Metadata.getInstance<Text>('text-CONTRATO-return').text = data.NRCONTRATOFCT;
            Metadata.getInstance<Text>('text-PARCEIRODENEGOCIO-return').text = data.NMFANTASIA;
            Metadata.getInstance<Text>('text-ESCOPODEFATURAMENTO-return').text = data.NMTIPOORCAMENTO;
            Metadata.getInstance<Text>('text-TIPODEFATURAMENTO-return').text = TIPODEFATURAMENTO;

            // Código que ficarão invisíveis na tela, mas são importantes para o update da verificação
            Metadata.getInstance<TextInput>('codigo').value = (data.IDTIPO === 'F') ? data.NRFATURACONT : data.NRMEDICAOFATURA;
            Metadata.getInstance<TextInput>('codigo-comparacao').value = (data.IDTIPO === 'F') ? data.NRFATURACONT_COMP : data.NRMEDICAOFATURA_COMP;
            Metadata.getInstance<TextInput>('idTipo').value = data.IDTIPO;

            // window.localStorage.removeItem('ROWDATA');
        }
    }

    public onMountedHeaderCard() {
        const rowdata = window.localStorage.getItem('ROWDATA');
        if (rowdata) {
            const data = JSON.parse(rowdata);
            const periodo = data.PERIODO ? data.PERIODO : 'não há dados';
            const periodoComp = data.PERIODO_COMP ? data.PERIODO_COMP : 'não há dados';

            const situacaoFat = this.traduction(data.IDSITUACAOFAT);
            const situacaoFatComp = this.traduction(data.IDSITUACAOFAT_COMP);

            const txSituacaoFat = situacaoFat ? ` - Situação: ${situacaoFat}` : '';
            const txSituacaoFatComp = situacaoFatComp ? ` - Situação: ${situacaoFatComp}` : '';

            Metadata.getInstance<Text>('title-card-firstPeriod').text = `Período de Apuração: ${periodo}${txSituacaoFat}`;
            Metadata.getInstance<Text>('title-card-secondPeriod').text = `Período (Comparação): ${periodoComp}${txSituacaoFatComp}`;

            // window.localStorage.removeItem('ROWDATA');
        }
    }

    public titleFirstPeriod: any;

    public titleSecondPeriod: any;

    public onMountedHeaderCardMeasurement() {
        const rowdata = window.localStorage.getItem('ROWDATA');
        if (rowdata) {
            const data = JSON.parse(rowdata);
            const periodo = data.PERIODO ? data.PERIODO : 'não há dados';
            const periodoComp = data.PERIODO_COMP ? data.PERIODO_COMP : 'não há dados';

            const situacaoMed = this.traductionSituacaoMed(data.IDSITUACAOMED);
            const situacaoMedComp = this.traductionSituacaoMed(data.IDSITUACAOMED_COMP);

            const txSituacaoMed = situacaoMed ? ` - Situação: ${situacaoMed}` : '';
            const txSituacaoMedComp = situacaoMedComp ? ` - Situação: ${situacaoMedComp}` : '';

            this.titleFirstPeriod = `Período de Apuração: ${periodo}${txSituacaoMed}`;
            this.titleSecondPeriod = `Período (Comparação): ${periodoComp}${txSituacaoMedComp}`;

            // window.localStorage.removeItem('ROWDATA');
        }
    }

    public traduction(value: any) {
        switch (value) {
            case 'A':
                return 'Periódico (valor de parcela fixo)';
            case 'X':
                return 'Periódico (Valor Apurado por Medição)';
            case 'P':
                return 'Parcelamento (Quantidade de Parcelas)';
            case '1':
                return 'Pendente';
            case '2':
                return 'Apurada';
            case '3':
                return 'Faturada';
            case '4':
                return 'Anulada';
            case '5':
                return 'Paralisada';
            case '6':
                return 'Parcialmente Apurada';
            default:
                return value;
        }
    }

    public traductionSituacaoMed(value: string) {
        switch (value) {
            case '1':
                return 'Pendente';
            case '3':
                return 'Calculada';
            case '4':
                return 'Parcela Gerada';
            case '5':
                return 'Paralisada';
            case '6':
                return 'Cancelada';
            default:
                return value;
        }
    }

    public reloadTabs() {
        this.onMountedHeaderBilling();

        if (this.visibilityParcel) {
            this.onMountedHeaderCard();
            this.mountFilterFrameParcelFirstPeriod();
            this.mountFilterFrameParcelSecondPeriod();
        }

        if (this.visibilityMeasurement) {
            this.onMountedHeaderCardMeasurement();
            setTimeout(() => {
                this.mountFilterMeasurementItemsFirstPeriod();
                this.mountFilterMeasurementItemsSecondPeriod();
            }, 1000);
        }
    }

    public mountFilterFrameParcelFirstPeriod() {
        const NRFATURACONT = Metadata.getInstance<Text>('text-NRFATURACONT').text;
        const grid = Metadata.getInstance<TekGrid>('grid-firstPeriod-frameParcel');

        this.setFilterGrid(grid, NRFATURACONT);
    }

    public mountFilterFrameParcelSecondPeriod() {
        const NRFATURACONT = Metadata.getInstance<Text>('text-NRFATURACONT_COMP').text;
        const grid = Metadata.getInstance<TekGrid>('grid-secondPeriod-frameParcel');

        this.setFilterGrid(grid, NRFATURACONT);
    }

    // Enviar o parâmetro NR para filtrar a requisição para montar o Grid do Primeiro Período da Tab de Medição
    public mountFilterMeasurementItemsFirstPeriod() {
        const NRMEDICAOFATURA = Metadata.getInstance<Text>('text-NRMEDICAOFATURA').text;
        const grid = Metadata.getInstance<TekGrid>('Grid-ContractsItems-firstPeriod');

        this.setFilterGrid(grid, NRMEDICAOFATURA).then(() => {
            [grid.datasource.currentRow] = grid.datasource.data;
            this.mountFilterMeasurementVariableFirstPeriod(grid.datasource.data[0]);
        });
    }

    public mountFilterMeasurementItemsSecondPeriod() {
        const NRMEDICAOFATURA = Metadata.getInstance<Text>('text-NRMEDICAOFATURA_COMP').text;
        const grid = Metadata.getInstance<TekGrid>('Grid-ContractsItems-secondPeriod');

        this.setFilterGrid(grid, NRMEDICAOFATURA).then(() => {
            [grid.datasource.currentRow] = grid.datasource.data;
            this.mountFilterMeasurementVariableSecondPeriod(grid.datasource.data[0]);
        });
    }

    public clickItemMedicaoFirstPeriod({ row }: IEventParam<TekGrid>) {
        const NRMEDICAOFATURA = Metadata.getInstance<Text>('text-NRMEDICAOFATURA').text.toString();
        if (NRMEDICAOFATURA.match(/_/g)?.length !== 2) {
            this.mountFilterMeasurementVariableFirstPeriod(row);
        }
    }

    public clickItemMedicaoSecondPeriod({ row }: IEventParam<TekGrid>) {
        const NRMEDICAOFATURA = Metadata.getInstance<Text>('text-NRMEDICAOFATURA_COMP').text.toString();
        if (NRMEDICAOFATURA.match(/_/g)?.length !== 2) {
            this.mountFilterMeasurementVariableSecondPeriod(row);
        }
    }

    public mountFilterMeasurementVariableFirstPeriod(row: IDictionary<any>) {
        const nrmedicaoItem = row ? row.NRMEDICAOFATURA : null;
        const gridMedicoes = Metadata.getInstance<TekGrid>('Grid-Element-Service-firstPeriod');

        this.setFilterGrid(gridMedicoes, nrmedicaoItem);
    }

    public mountFilterMeasurementVariableSecondPeriod(row: IDictionary<any>) {
        const nrmedicaoItem = row ? row.NRMEDICAOFATURA : null;
        const gridMedicoes = Metadata.getInstance<TekGrid>('Grid-Element-Service-secondPeriod');

        this.setFilterGrid(gridMedicoes, nrmedicaoItem);
    }

    public setFilterGrid(grid: TekGrid, NR: any) {
        const codigo = NR || -1;
        return grid.datasource.setFilter({
            NR: codigo,
        });
        // grid.datasource.get();
    }

    public getMascaraCompararCom(value: string) {
        if (value.replace('º', '').length <= 1) return '#º';
        return '##º';
    }
}
