import { Utilities } from '@/util/Utilities';
import {
    Button, Modal, ModalService, Select, Tabs, TextInput, Text, LoadingService, Form, DialogService,
} from '@zeedhi/common';
import { Http, Metadata } from '@zeedhi/core';
import { TekGrid } from '@zeedhi/teknisa-components-common';

export class Sm00004006DocumentoContratoController {
    public tekgridDocumentoContratoVisivel: boolean = true;

    public currentRow: any = {};

    private basicModal!: Modal;

    get formDocumentoContratoVisivel() {
        return !this.tekgridDocumentoContratoVisivel;
    }

    public voltarTekGridDocumentoContrato() {
        this.tekgridDocumentoContratoVisivel = true;
    }

    get textFrameDocumentoContrato() {
        return this.currentRow.NMCONTRATO;
    }

    public abrirFrameDocumentoContrato({ row }: any) {
        this.tekgridDocumentoContratoVisivel = false;
        this.currentRow = row;
        const tabs = Metadata.getInstance<Tabs>('tabsInformationDocumentoContrato');
        tabs.activeTab = 0;
        this.carregarTabsDocumentoContrato(tabs);
    }

    public carregarTabsDocumentoContrato(tabs: any) {
        const tabAtiva = tabs.activeTab;

        if (tabs.tabs[tabAtiva].name === 'tab_documentoContrato') {
            this.carregarDadosAbaDocumentoContrato();
        }

        if (tabs.tabs[tabAtiva].name === 'tab_anexosDocumentoContrato') {
            this.carregarDadosAbaAnexoDocumentoContrato();
        }
    }

    public carregarDadosAbaDocumentoContrato() {
        Metadata.getInstance<TextInput>('formDadosContratoDocumentoContrato-NMCONTRATO').value = this.currentRow.NMCONTRATO;
        Metadata.getInstance<TextInput>('formDadosContratoDocumentoContrato-NMPRINCIPALPARC').value = this.currentRow.NMPRINCIPALPARC;
        const textIDSITUACAOATUALCONTRATO = this.currentRow.IDSITUACAOATUALCONTRATO;
        switch (textIDSITUACAOATUALCONTRATO) {
            case 'P':
                Metadata.getInstance<TextInput>('formDadosContratoDocumentoContrato-IDSITUACAOATUALCONTRATO').value = 'Pendente';
                break;
            case 'A':
                Metadata.getInstance<TextInput>('formDadosContratoDocumentoContrato-IDSITUACAOATUALCONTRATO').value = 'Aprovado';
                break;
            case 'R':
                Metadata.getInstance<TextInput>('formDadosContratoDocumentoContrato-IDSITUACAOATUALCONTRATO').value = 'Reprovado';
                break;
            case 'E':
                Metadata.getInstance<TextInput>('formDadosContratoDocumentoContrato-IDSITUACAOATUALCONTRATO').value = 'Encerrado';
                break;
            case 'D':
                Metadata.getInstance<TextInput>('formDadosContratoDocumentoContrato-IDSITUACAOATUALCONTRATO').value = 'Documento Enviado';
                break;
            default:
                break;
        }
        Metadata.getInstance<TextInput>('formDadosContratoDocumentoContrato-DTINICONTRATO').value = this.currentRow.DTINICONTRATO;
        Metadata.getInstance<TextInput>('formDadosContratoDocumentoContrato-DTFINCONTRATO').value = this.currentRow.DTFINCONTRATO;
        Metadata.getInstance<TextInput>('formDadosDocumentoContrato-DSMODELODOCORCA').value = this.currentRow.DSMODELODOCORCA;
        Metadata.getInstance<Select>('formDadosDocumentoContrato-IDPADRAO').value = this.currentRow.IDPADRAO;
        Metadata.getInstance<TextInput>('formDadosDocumentoContrato-VRTAMANHOLOGO').value = this.currentRow.VRTAMANHOLOGO;
        Metadata.getInstance<Select>('formDadosDocumentoContrato-IDSITUACAOCONTRATO').value = this.currentRow.IDSITUACAOCONTRATO;
    }

    public carregarDadosAbaAnexoDocumentoContrato() {
        const nrseqcontdoc = this.currentRow.NRSEQCONTDOC;
        try {
            const grid = Metadata.getInstance<TekGrid>('basic-tek-grid-tab-anexos-documento-contrato');
            grid.datasource.setFilter({
                NRSEQCONTDOC: nrseqcontdoc,
            });
        } catch (e) {
            // console.error(e);
        }
    }

    public openFilterDocumentoContrato({ component }: any) {
        const filterButton = Metadata.getInstance<Button>(`${component.name}_filterButton`);
        filterButton.click({} as Event, {} as HTMLElement);
    }

    public fecharModalInsercaoDocumentoContrato() {
        this.basicModal.hide();
        this.basicModal?.destroy();
    }

    public async abrirModalInsercaoDocumentoContrato() {
        await this.exibirModalInsercaoDocumentoContrato();
        Metadata.getInstance<Text>('titleText').text = 'Novo Documento de Contrato';
    }

    public async exibirModalInsercaoDocumentoContrato() {
        this.basicModal = await ModalService.createFromJson('sm_00004_006_documento_contrato/modal/sm_00004_006_insercao_documento_contrato', true);
        this.basicModal.show();
    }

    public setFilterCamposModalInsert({ component }: any) {
        const nrseqcontrato = this.currentRow.NRSEQCONTRATO;
        try {
            component.datasource.setFilter({
                NRSEQCONTRATO: nrseqcontrato,
            });
        } catch (e) {
            console.error(e);
        }
    }

    public setModalInsertDocumentoContrato({ component }: any) {
        const nrseqcontrato = component.value;
        component.datasource.data.forEach((dados: any) => {
            if (dados.NRSEQCONTRATO === nrseqcontrato || nrseqcontrato !== null) {
                Metadata.getInstance<TextInput>('formDadosContratoModalDocumentoContrato-NMPRINCIPALPARC').value = dados.NMPRINCIPALPARC;
                const textSituacao = dados.IDSITUACAOATUALCONTRATO;
                switch (textSituacao) {
                    case 'P':
                        Metadata.getInstance<TextInput>('formDadosContratoModalDocumentoContrato-IDSITUACAOATUALCONTRATO').value = 'Pendente';
                        break;
                    case 'A':
                        Metadata.getInstance<TextInput>('formDadosContratoModalDocumentoContrato-IDSITUACAOATUALCONTRATO').value = 'Aprovado';
                        break;
                    case 'R':
                        Metadata.getInstance<TextInput>('formDadosContratoModalDocumentoContrato-IDSITUACAOATUALCONTRATO').value = 'Reprovado';
                        break;
                    case 'E':
                        Metadata.getInstance<TextInput>('formDadosContratoModalDocumentoContrato-IDSITUACAOATUALCONTRATO').value = 'Encerrado';
                        break;
                    case 'D':
                        Metadata.getInstance<TextInput>('formDadosContratoModalDocumentoContrato-IDSITUACAOATUALCONTRATO').value = 'Documento Enviado';
                        break;
                    default:
                        break;
                }
                Metadata.getInstance<TextInput>('formDadosContratoModalDocumentoContrato-DTINICONTRATO').value = dados.DTINICONTRATO;
                Metadata.getInstance<TextInput>('formDadosContratoModalDocumentoContrato-DTFINCONTRATO').value = dados.DTFINCONTRATO;
            } else {
                Metadata.getInstance<TextInput>('formDadosContratoModalDocumentoContrato-NMPRINCIPALPARC').value = null;
                Metadata.getInstance<TextInput>('formDadosContratoModalDocumentoContrato-IDSITUACAOATUALCONTRATO').value = null;
                Metadata.getInstance<TextInput>('formDadosContratoModalDocumentoContrato-DTINICONTRATO').value = null;
                Metadata.getInstance<TextInput>('formDadosContratoModalDocumentoContrato-DTFINCONTRATO').value = null;
            }
        });
    }

    public salvarDocumentoContrato() {
        const formContrato = Metadata.getInstance<Form>('formDadosContratoModalDocumentoContrato');
        const formDocumento = Metadata.getInstance<Form>('formDadosDocumentoModalContrato');
        const grid = Metadata.getInstance<TekGrid>('tek-grid-documento_contrato_grid');
        const params = {
            NRSEQCONTRATO: Metadata.getInstance<Select>('formDadosContratoModalDocumentoContrato-NMCONTRATO').value,
            IDPADRAO: Metadata.getInstance<Select>('formDadosDocumentoModalContrato-IDPADRAO').value,
            VRTAMANHOLOGO: Metadata.getInstance<TextInput>('formDadosDocumentoModalContrato-VRTAMANHOLOGO').value,
            IDSITUACAOCONTRATO: Metadata.getInstance<Select>('formDadosDocumentoModalContrato-IDSITUACAOCONTRATO').value,
            DSMODELODOCORCA: Metadata.getInstance<Select>('formDadosDocumentoModalContrato-DSMODELODOCORCA').value,
        };
        if (Utilities.validateForm(formContrato) && Utilities.validateForm(formDocumento)) {
            LoadingService.show();
            Http.post('/inserirDocumentoContrato', params).then(() => {
                this.fecharModalInsercaoDocumentoContrato();
                grid.datasource.get();
                Utilities.showSuccessMessage();
            }).catch((error: any) => {
                Utilities.showErrorMessage(error);
            }).finally(() => {
                LoadingService.hide();
            });
        }
    }

    public showConfirmDeleteMessage({ event }: any) {
        event!.preventDefault();
        DialogService.show({
            name: 'buttons-dialog-documento-contrato',
            type: 'warning',
            title: 'Excluir Registro',
            text: 'Deseja realmente excluir todos os registros selecionados?',
            buttons: [
                {
                    name: 'NAO',
                    component: 'ZdButton',
                    label: 'Não',
                    outline: true,
                    events: {
                        click: () => {
                            DialogService.hide();
                        },
                    },
                },
                {
                    name: 'SIM',
                    component: 'ZdButton',
                    label: 'Sim',
                    events: {
                        click: () => {
                            this.deleteDocumentoContrato();
                            DialogService.hide();
                        },
                    },
                },
            ],
        });
    }

    public deleteDocumentoContrato() {
        const grid = Metadata.getInstance<TekGrid>('tek-grid-documento_contrato_grid');
        LoadingService.show();
        Http.post('/deleteDocumentoContrato', grid.selectedRows)
            .then(() => {
                grid.selectedRows = [];
                grid.reload();
                Utilities.showSuccessMessage();
            }).catch((error) => {
                Utilities.showErrorMessage(error);
            }).finally(() => {
                LoadingService.hide();
            });
    }
}
